library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_unsigned.all;

entity Command_generator is
port(
		CLK				: in std_logic;
		ENABLE			: in std_logic;
		RES				: in std_logic;
		WR_DATA			: out std_logic_vector(127 downto 0);
		WR_EN				: out std_logic;
		WR_FULL			: in std_logic;
		WR_EMPTY			: in std_logic;
		WR_COUNT			: in std_logic_vector(6 downto 0);
		ADRESSE			: out std_logic_vector(29 downto 0);
		COMMAND			: out std_logic_vector(2 downto 0);
		BURST_LAENGE 	: out std_logic_vector(5 downto 0);
		MASKE				: out std_logic_vector(15 downto 0);
		CMD_EN			: out std_logic;
		CMD_EMPTY		: in std_logic;
		CMD_FULL			: in std_logic;
		RD_DATA			: in std_logic_vector(127 downto 0);
		RD_EN				: out std_logic;
		RD_COUNT			: in std_logic_vector(6 downto 0);
		RD_FULL			: in std_logic;
		RD_EMPTY			: in std_logic;
		LED_BLINK		: out std_logic;
		LED_DDR2			: out std_logic;
		LED_CALIB		: out std_logic;
		CALIB_DONE		: in std_logic);

end Command_generator;

architecture Behavioral of Command_generator is

	type ZUSTAENDE is (Z0, Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10);
	signal ZUSTAND : ZUSTAENDE;
	signal ZAEHLER : std_logic_vector(3 downto 0) := "0000";
	signal c : integer range 0 to 24999999 := 0; -- 1s bei 25MHz fosc
	signal x : std_logic:= '0'; 	
	
begin
	
	DDR2_PROCESS : process(CLK)
		begin
			if(rising_edge (CLK)) then

				if(ENABLE = '1') then
					ZUSTAND <= Z0;
					
					if(CALIB_DONE = '1') then
						LED_CALIB <= '1';
						
						case ZUSTAND is
							--write DATA in FIFO
							when	Z0 =>	WR_DATA <= "10000000110000001110000011110000111110001111110011111110111111111000000011000000111000001111000011111000111111001111111011111111";
											WR_EN <= '0';
											CMD_EN <= '0';
											RD_EN <= '0';
											ZUSTAND <= Z1;
							when 	Z1 => WR_EN <= '1';
											ZUSTAND <= Z2;
							WHEN	Z2 => WR_EN <= '0';
											ZUSTAND <= Z3;
							--write FIFO in DDR2
							WHEN	Z3 => ADRESSE <= "100000000000000000000000000000";
											BURST_LAENGE <= "000000";
											MASKE <= "0000000000000001";
											COMMAND <= "000";
											ZUSTAND <= Z4;
							WHEN	Z4 => if(ZAEHLER = "0010") then
												CMD_EN <= '1';
												ZUSTAND <= Z5;
												ZAEHLER <= "0000";
											else 	ZAEHLER <= ZAEHLER + "0001";
													ZUSTAND <= Z4;
											end if;
							WHEN	Z5 => CMD_EN <= '0';
											ZUSTAND <= Z6;
											ADRESSE <= "100000000000000000000000000000";
											BURST_LAENGE <= "000000";
							--read FIFO from DDR2
							WHEN	Z6 => if(ZAEHLER = "0010") then
												COMMAND <= "001";
												ZUSTAND <= Z7;
												ZAEHLER <= "0000";
											else 	ZAEHLER <= ZAEHLER + "0001";
													ZUSTAND <= Z6;
											end if;
							WHEN	Z7 => CMD_EN <= '1';
											ZUSTAND <= Z8;
							WHEN	Z8 => CMD_EN <= '0';
											ZUSTAND <= Z9;
							--read DATA from FIFO
							WHEN	Z9 => if(ZAEHLER = "1111") then
												ZUSTAND <= Z10;
												ZAEHLER <= "0000";
												RD_EN <= '1';
											else 	ZAEHLER <= ZAEHLER + "0001";
													ZUSTAND <= Z9;
											end if;
							WHEN	Z10 => RD_EN <= '0';
											 ZUSTAND <= Z0;
							when others => null;
						end case;
					
					else
						LED_CALIB <= '0';
					end if;

				else	WR_DATA <= "10000000110000001110000011110000111110001111110011111110111111111000000011000000111000001111000011111000111111001111111011111111";
						WR_EN <= '0';
						RD_EN <= '0';
						CMD_EN <= '0';
						ZUSTAND <= Z0;
						ADRESSE <= "000000000000000000000000000000";
						BURST_LAENGE <= "000000";
						COMMAND <= "000";
						ZAEHLER <= "0000";
						MASKE <= "0000000000000000";
				end if;
				
				if(RD_DATA = "10000000110000001110000011110000111110001111110011111110111111111000000011000000111000001111000011111000111111001111111011111111") then
					LED_DDR2 <= '1';
				else
					LED_DDR2 <= '0';
				end if;
				
			end if;
	end process;

   process 
		begin  
			wait until rising_edge(CLK); 
				if (c < 24999999) then         
					 c <= c + 1;               
				else                        
					 c <= 0;                  
					 x <= not x;             
				end if; 
   end process; 
	
   LED_BLINK <= x; 
	
end Behavioral;

