library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity DDR2_Test_Top is
	port(
			CLK_P			: in std_logic;
			CLK_N			: in std_logic;
			ENABLE		: in std_logic;
			RES			: in std_logic;
			
			LED_BLINK	: out std_logic;
			LED_DDR2		: out std_logic;
			LED_CALIB	: out std_logic;
			
			--MCB3_CK		: out std_logic;
			--MCB3_CK_N	: out std_logic;
			--MCB3_WE_N	: out std_logic;

			INTF_RST		: out std_logic;
			INTF_WR_EN	: out std_logic;
			INTF_WR_ER	: out std_logic;
			INTF_WR_EM	: out std_logic);
end DDR2_Test_Top;

architecture Behavioral of DDR2_Test_Top is
				
	component Command_generator
		port(
			CLK				: in std_logic;
			ENABLE			: in std_logic;
			RES				: in std_logic;
			WR_DATA			: out std_logic_vector(127 downto 0);
			WR_EN				: out std_logic;
			WR_FULL			: in std_logic;
			WR_EMPTY			: in std_logic;
			WR_COUNT			: in std_logic_vector(6 downto 0);
			ADRESSE			: out std_logic_vector(29 downto 0);
			COMMAND			: out std_logic_vector(2 downto 0);
			BURST_LAENGE 	: out std_logic_vector(5 downto 0);
			MASKE				: out std_logic_vector(15 downto 0);
			CMD_EN			: out std_logic;
			CMD_EMPTY		: in std_logic;
			CMD_FULL			: in std_logic;
			RD_DATA			: in std_logic_vector(127 downto 0);
			RD_EN				: out std_logic;
			RD_COUNT			: in std_logic_vector(6 downto 0);
			RD_FULL			: in std_logic;
			RD_EMPTY			: in std_logic;
			LED_BLINK		: out std_logic;
			LED_DDR2			: out std_logic;
			LED_CALIB		: out std_logic;
			CALIB_DONE		: in std_logic);
	end component;

component DDR2_MIC
	generic(
		C3_P0_MASK_SIZE           : integer := 16;
		C3_P0_DATA_PORT_SIZE      : integer := 128;
		C3_MEMCLK_PERIOD          : integer := 5000;
		C3_RST_ACT_LOW            : integer := 0;
		C3_INPUT_CLK_TYPE         : string := "DIFFERENTIAL";
		C3_CALIB_SOFT_IP          : string := "TRUE";
		C3_SIMULATION             : string := "FALSE";
		DEBUG_EN                  : integer := 0;
		C3_MEM_ADDR_ORDER         : string := "BANK_ROW_COLUMN";
		C3_NUM_DQ_PINS            : integer := 16;
		C3_MEM_ADDR_WIDTH         : integer := 13;
		C3_MEM_BANKADDR_WIDTH     : integer := 3);
	
	port (
		mcb3_dram_dq              : inout  std_logic_vector(C3_NUM_DQ_PINS-1 downto 0);
		mcb3_dram_a               : out std_logic_vector(C3_MEM_ADDR_WIDTH-1 downto 0);
		mcb3_dram_ba              : out std_logic_vector(C3_MEM_BANKADDR_WIDTH-1 downto 0);
		mcb3_dram_ras_n           : out std_logic;
		mcb3_dram_cas_n           : out std_logic;
		mcb3_dram_we_n            : out std_logic;
		mcb3_dram_odt             : out std_logic;
		mcb3_dram_cke             : out std_logic;
		mcb3_dram_dm              : out std_logic;
		mcb3_dram_udqs            : inout  std_logic;
		mcb3_rzq                  : inout  std_logic;
		--mcb3_zio                  : inout  std_logic;
		mcb3_dram_udm             : out std_logic;
		c3_sys_clk_p              : in  std_logic;
		c3_sys_clk_n              : in  std_logic;
		c3_sys_rst_i              : in  std_logic;
		c3_calib_done             : out std_logic;
		c3_clk0                   : out std_logic;
		c3_rst0                   : out std_logic;
		mcb3_dram_dqs             : inout  std_logic;
		mcb3_dram_ck              : out std_logic;
		mcb3_dram_ck_n            : out std_logic;
		c3_p0_cmd_clk             : in std_logic;
		c3_p0_cmd_en              : in std_logic;
		c3_p0_cmd_instr           : in std_logic_vector(2 downto 0);
		c3_p0_cmd_bl              : in std_logic_vector(5 downto 0);
		c3_p0_cmd_byte_addr       : in std_logic_vector(29 downto 0);
		c3_p0_cmd_empty           : out std_logic;
		c3_p0_cmd_full            : out std_logic;
		c3_p0_wr_clk              : in std_logic;
		c3_p0_wr_en               : in std_logic;
		c3_p0_wr_mask             : in std_logic_vector(C3_P0_MASK_SIZE - 1 downto 0);
		c3_p0_wr_data             : in std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0);
		c3_p0_wr_full             : out std_logic;
		c3_p0_wr_empty            : out std_logic;
		c3_p0_wr_count            : out std_logic_vector(6 downto 0);
		c3_p0_wr_underrun         : out std_logic;
		c3_p0_wr_error            : out std_logic;
		c3_p0_rd_clk              : in std_logic;
		c3_p0_rd_en               : in std_logic;
		c3_p0_rd_data             : out std_logic_vector(C3_P0_DATA_PORT_SIZE - 1 downto 0);
		c3_p0_rd_full             : out std_logic;
		c3_p0_rd_empty            : out std_logic;
		c3_p0_rd_count            : out std_logic_vector(6 downto 0);
		c3_p0_rd_overflow         : out std_logic;
		c3_p0_rd_error            : out std_logic);
end component;

   signal mcb3_dram_dq 			: std_logic_vector(15 downto 0);
   signal mcb3_dram_udqs 		: std_logic;
   signal mcb3_dram_udqs_n 	: std_logic;
   signal rzq3 					: std_logic;
	--signal zio3 					: std_logic;
   signal mcb3_dram_dqs 		: std_logic;
   signal mcb3_dram_dqs_n 		: std_logic;
   signal mcb3_dram_a 			: std_logic_vector(12 downto 0);
   signal mcb3_dram_ba 			: std_logic_vector(2 downto 0);
   signal mcb3_dram_ras_n 		: std_logic;
   signal mcb3_dram_cas_n 		: std_logic;
   signal mcb3_dram_we_n 		: std_logic;
   signal mcb3_dram_odt 		: std_logic;
   signal mcb3_dram_cke 		: std_logic;
   signal mcb3_dram_dm 			: std_logic;
   signal mcb3_dram_udm 		: std_logic;
	signal mcb3_dram_ck 			: std_logic;
   signal mcb3_dram_ck_n 		: std_logic;

   signal c3_clk0_i 				: std_logic;
   signal c3_rst0 				: std_logic;
	
   signal c3_p0_wr_underrun 	: std_logic;
   signal c3_p0_rd_overflow 	: std_logic;
   signal c3_p0_rd_error 		: std_logic;
	
	signal CALIB_DONE_i			: std_logic;
	
	signal WR_EN_i 			: std_logic;
	signal MASKE_i 			: std_logic_vector(15 downto 0);
	signal WR_DATA_i 			: std_logic_vector(127 downto 0);
	signal ADRESSE_i 			: std_logic_vector(29 downto 0);
	signal COMMAND_i 			: std_logic_vector(2 downto 0);
	signal BURST_LAENGE_i 	: std_logic_vector(5 downto 0);
	signal WR_FULL_i			: std_logic;
	signal WR_EMPTY_i			: std_logic;
	signal WR_COUNT_i			: std_logic_vector(6 downto 0);
	signal CMD_EN_i			: std_logic;
	signal CMD_EMPTY_i		: std_logic;
	signal CMD_FULL_i			: std_logic;
	signal RD_DATA_i			: std_logic_vector(127 downto 0);
	signal RD_EN_i				: std_logic;
	signal RD_COUNT_i			: std_logic_vector(6 downto 0);
	signal RD_FULL_i			: std_logic;
	signal RD_EMPTY_i			: std_logic;
	signal WR_ERROR_i  		: std_logic;
	
	begin	
	
	--MCB3_CKE <= mcb3_dram_cke;
	--MCB3_CK <= mcb3_dram_ck;
	--MCB3_CK_N <= mcb3_dram_ck_n;
	
	INTF_RST <= c3_rst0;
	INTF_WR_EN <= WR_EN_i;
	INTF_WR_ER <= WR_ERROR_i;
	INTF_WR_EM <= WR_EMPTY_i;
	
	Inst_DDR2_MIC : DDR2_MIC
		port map(
			c3_sys_clk_p  	=>	CLK_P,
			c3_sys_clk_n   => CLK_N,
			c3_sys_rst_i	=>	RES,                        

			mcb3_dram_dq		=>    mcb3_dram_dq,  
			mcb3_dram_a			=>    mcb3_dram_a,  
			mcb3_dram_ba		=>    mcb3_dram_ba,
			mcb3_dram_ras_n	=>    mcb3_dram_ras_n,                        
			mcb3_dram_cas_n	=>    mcb3_dram_cas_n,                        
			mcb3_dram_we_n		=>    mcb3_dram_we_n,                          
			mcb3_dram_odt    	=>    mcb3_dram_odt,
			mcb3_dram_cke		=>    mcb3_dram_cke,                          
			mcb3_dram_ck		=>    mcb3_dram_ck,                          
			mcb3_dram_ck_n		=>    mcb3_dram_ck_n,       
			mcb3_dram_dqs		=>    mcb3_dram_dqs,                          
			mcb3_dram_udqs		=>    mcb3_dram_udqs,    -- for X16 parts           
			mcb3_dram_udm		=>    mcb3_dram_udm,     -- for X16 parts
			mcb3_dram_dm  		=>    mcb3_dram_dm,
			c3_clk0				=>	   c3_clk0_i,
			c3_rst0				=>    c3_rst0,
		
			c3_calib_done     =>    CALIB_DONE_i,
			mcb3_rzq        	=>    rzq3,
			--mcb3_zio        	=>    zio3,
			
			c3_p0_cmd_clk        =>  c3_clk0_i,
			c3_p0_cmd_en         =>  CMD_EN_i,
			c3_p0_cmd_instr      =>  COMMAND_i,
			c3_p0_cmd_bl         =>  BURST_LAENGE_i,
			c3_p0_cmd_byte_addr	=>  ADRESSE_i,
			c3_p0_cmd_empty      =>  CMD_EMPTY_i,
			c3_p0_cmd_full       =>  CMD_FULL_i,
			c3_p0_wr_clk         =>  c3_clk0_i,
			c3_p0_wr_en          =>  WR_EN_i,
			c3_p0_wr_mask        =>  MASKE_i,
			c3_p0_wr_data        =>  WR_DATA_i,
			c3_p0_wr_full        =>  WR_FULL_i,
			c3_p0_wr_empty       =>  WR_EMPTY_i,
			c3_p0_wr_count       =>  WR_COUNT_i,
			c3_p0_wr_underrun    =>  c3_p0_wr_underrun,
			c3_p0_wr_error       =>  WR_ERROR_i,
			c3_p0_rd_clk         =>  c3_clk0_i,
			c3_p0_rd_en          =>  RD_EN_i,
			c3_p0_rd_data        =>  RD_DATA_i,
			c3_p0_rd_full        =>  RD_FULL_i,
			c3_p0_rd_empty       =>  RD_EMPTY_i,
			c3_p0_rd_count       =>  RD_COUNT_i,
			c3_p0_rd_overflow    =>  c3_p0_rd_overflow,
			c3_p0_rd_error       =>  c3_p0_rd_error);

	Inst_Command_generator: Command_generator 
		port map(
			CLK 				=> c3_clk0_i,
			RES 				=> RES,
			WR_DATA 			=> WR_DATA_i,
			ADRESSE 			=> ADRESSE_i,
			COMMAND 			=> COMMAND_i,
			BURST_LAENGE 	=> BURST_LAENGE_i,
			MASKE 			=> MASKE_i,
			WR_EN 			=> WR_EN_i,
			WR_FULL 			=> WR_FULL_i,
			WR_EMPTY 		=> WR_EMPTY_i,
			WR_COUNT 		=> WR_COUNT_i,
			CMD_EN 			=> CMD_EN_i,
			CMD_EMPTY		=> CMD_EMPTY_i,
			CMD_FULL 		=> CMD_FULL_i,
			ENABLE 			=> ENABLE,
			RD_EN 			=> RD_EN_i,
			RD_DATA 			=> RD_DATA_i,
			RD_FULL 			=> RD_FULL_i,
			RD_EMPTY 		=> RD_EMPTY_i,
			RD_COUNT 		=> RD_COUNT_i,
			LED_BLINK		=> LED_BLINK,
			LED_DDR2			=> LED_DDR2,
			LED_CALIB		=> LED_CALIB,
			CALIB_DONE     => CALIB_DONE_i);

end Behavioral;